// Button.java
// Sends ButtonEvents to ButtonListeners when accessed
package com.deitel.jhtp4.elevator.model;

// Deitel packages
import com.deitel.jhtp4.elevator.event.*;

public class Button implements ElevatorMoveListener {

   // ButtonListener
   private ButtonListener buttonListener = null;

   // represent whether Button is pressed
   private boolean pressed = false;

   // set listener
   public void setButtonListener( ButtonListener listener )
   {
      buttonListener = listener;
   }

   // press Button and send ButtonEvent
   public void pressButton( Location location )
   {
      pressed = true;

      buttonListener.buttonPressed( 
         new ButtonEvent( this, location ) );
   }

   // reset Button and send ButtonEvent
   public void resetButton( Location location )
   {
      pressed = false;

      buttonListener.buttonReset( 
         new ButtonEvent( this, location ) );
   }

   // return whether button is pressed
   public boolean isButtonPressed()
   {
      return pressed;
   }

   // invoked when Elevator has departed
   public void elevatorDeparted( ElevatorMoveEvent moveEvent ) {}

   // invoked when Elevator has arrived
   public void elevatorArrived( ElevatorMoveEvent moveEvent )
   {
      resetButton( moveEvent.getLocation() );
   }
}